@@ -324,9 +324,9 @@ def test_qda_deprecation():
                          "removed in 0.21.", clf.fit, X, y)
 
     # check that covariance_ (and covariances_ with warning) is stored
-    assert_warns_message(DeprecationWarning, "Attribute covariances_ was "
+    assert_warns_message(DeprecationWarning, "Attribute ``covariances_`` was "
                          "deprecated in version 0.19 and will be removed "
-                         "in 0.21. Use covariance_ instead", getattr, clf,
+                         "in 0.21. Use ``covariance_`` instead", getattr, clf,
                          'covariances_')
 
 
